"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Create = void 0;
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const AndroidEnvironmentRequirements_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/AndroidEnvironmentRequirements");
const AndroidUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/AndroidUtils");
const Common_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/Common");
const CommonUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/CommonUtils");
const IOSEnvironmentRequirements_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/IOSEnvironmentRequirements");
const IOSUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/IOSUtils");
const Requirements_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/Requirements");
const util_1 = __importDefault(require("util"));
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/lwc-dev-mobile', 'device-create');
class Create extends command_1.SfdxCommand {
    constructor() {
        super(...arguments);
        this.examples = [
            `sfdx force:lightning:local:device:create -p iOS -n MyNewVirtualDevice -d iPhone-8`,
            `sfdx force:lightning:local:device:create -p Android -n MyNewVirtualDevice -d pixel_xl`
        ];
        this.platform = '';
        this.deviceName = '';
        this.deviceType = '';
        this._requirements = {};
    }
    async run() {
        try {
            await this.init();
        }
        catch (error) {
            if (error instanceof core_1.SfdxError) {
                const sfdxError = error;
                sfdxError.actions = this.examples;
                throw sfdxError;
            }
            throw error;
        }
        this.logger.info(`Device Create command invoked for ${this.flags.platform}`);
        this.platform = this.flags.platform;
        this.deviceName = this.flags.devicename;
        this.deviceType = this.flags.devicetype;
        return Requirements_1.RequirementProcessor.execute(this.commandRequirements)
            .then(() => {
            this.logger.info('Setup requirements met, continuing with Device Create');
            CommonUtils_1.CommonUtils.startCliAction(messages.getMessage('deviceCreateAction'), util_1.default.format(messages.getMessage('deviceCreateStatus'), this.deviceName, this.deviceType));
            return this.executeDeviceCreate();
        })
            .then(() => {
            const message = util_1.default.format(messages.getMessage('deviceCreateSuccessStatus'), this.deviceName, this.deviceType);
            CommonUtils_1.CommonUtils.stopCliAction(message);
        })
            .catch((error) => {
            CommonUtils_1.CommonUtils.stopCliAction(messages.getMessage('deviceCreateFailureStatus'));
            this.logger.warn(`Device Create failed for ${this.flags.platform}.`);
            return Promise.reject(error);
        });
    }
    async init() {
        if (this.logger) {
            return Promise.resolve();
        }
        return super
            .init()
            .then(() => core_1.Logger.child('force:lightning:local:device:create', {}))
            .then((logger) => {
            this.logger = logger;
            return Promise.resolve();
        });
    }
    get commandRequirements() {
        if (Object.keys(this._requirements).length === 0) {
            const requirements = {};
            requirements.setup = Common_1.CommandLineUtils.platformFlagIsAndroid(this.flags.platform)
                ? new AndroidEnvironmentRequirements_1.AndroidEnvironmentRequirements(this.logger, this.flags.apilevel)
                : new IOSEnvironmentRequirements_1.IOSEnvironmentRequirements(this.logger);
            requirements.create = {
                requirements: [
                    new DeviceNameAvailableRequirement(this, this.logger),
                    new ValidDeviceTypeRequirement(this, this.logger)
                ],
                enabled: true
            };
            this._requirements = requirements;
        }
        return this._requirements;
    }
    async executeDeviceCreate() {
        const isAndroid = Common_1.CommandLineUtils.platformFlagIsAndroid(this.platform);
        return isAndroid
            ? this.executeAndroidDeviceCreate()
            : this.executeIOSDeviceCreate();
    }
    async executeAndroidDeviceCreate() {
        return AndroidUtils_1.AndroidUtils.fetchSupportedEmulatorImagePackage(this.flags.apilevel).then((preferredPack) => {
            const emuImage = preferredPack.platformEmulatorImage || 'default';
            const androidApi = preferredPack.platformAPI;
            const abi = preferredPack.abi;
            return AndroidUtils_1.AndroidUtils.createNewVirtualDevice(this.deviceName, emuImage, androidApi, this.deviceType, abi);
        });
    }
    async executeIOSDeviceCreate() {
        return IOSUtils_1.IOSUtils.getSupportedRuntimes().then((supportedRuntimes) => IOSUtils_1.IOSUtils.createNewDevice(this.deviceName, this.deviceType, supportedRuntimes[0]));
    }
}
exports.Create = Create;
Create.description = messages.getMessage('commandDescription');
Create.flagsConfig = {
    devicename: command_1.flags.string({
        char: 'n',
        description: messages.getMessage('deviceNameFlagDescription'),
        required: true,
        validate: (deviceName) => {
            if (deviceName && deviceName.trim().length > 0) {
                return true;
            }
            else {
                throw new core_1.SfdxError(messages.getMessage('error:invalidDeviceNameFlagsDescription'), 'lwc-dev-mobile');
            }
        }
    }),
    devicetype: command_1.flags.string({
        char: 'd',
        description: messages.getMessage('deviceTypeFlagDescription'),
        required: true,
        validate: (deviceType) => {
            if (deviceType && deviceType.trim().length > 0) {
                return true;
            }
            else {
                throw new core_1.SfdxError(messages.getMessage('error:invalidDeviceTypeFlagsDescription'), 'lwc-dev-mobile');
            }
        }
    }),
    ...Common_1.CommandLineUtils.createFlagConfig(Common_1.FlagsConfigType.ApiLevel, false),
    ...Common_1.CommandLineUtils.createFlagConfig(Common_1.FlagsConfigType.Platform, true)
};
class DeviceNameAvailableRequirement {
    constructor(owner, logger) {
        this.title = messages.getMessage('reqs:devicename:title');
        this.fulfilledMessage = messages.getMessage('reqs:devicename:fulfilledMessage');
        this.unfulfilledMessage = messages.getMessage('reqs:devicename:unfulfilledMessage');
        this.owner = owner;
        this.logger = logger;
    }
    async checkFunction() {
        const isAndroid = Common_1.CommandLineUtils.platformFlagIsAndroid(this.owner.platform);
        const deviceName = this.owner.deviceName;
        const deviceAlreadyExists = isAndroid
            ? await AndroidUtils_1.AndroidUtils.hasEmulator(deviceName)
            : (await IOSUtils_1.IOSUtils.getSimulator(deviceName)) != null;
        return deviceAlreadyExists
            ? Promise.reject(util_1.default.format(this.unfulfilledMessage, deviceName))
            : Promise.resolve(util_1.default.format(this.fulfilledMessage, deviceName));
    }
}
class ValidDeviceTypeRequirement {
    constructor(owner, logger) {
        this.title = messages.getMessage('reqs:devicetype:title');
        this.fulfilledMessage = messages.getMessage('reqs:devicetype:fulfilledMessage');
        this.unfulfilledMessage = messages.getMessage('reqs:devicetype:unfulfilledMessage');
        this.owner = owner;
        this.logger = logger;
    }
    async checkFunction() {
        const isAndroid = Common_1.CommandLineUtils.platformFlagIsAndroid(this.owner.platform);
        const deviceType = this.owner.deviceType;
        const supportedDevices = isAndroid
            ? await AndroidUtils_1.AndroidUtils.getSupportedDevices()
            : await IOSUtils_1.IOSUtils.getSupportedDevices();
        const match = supportedDevices.find((device) => device === deviceType);
        return match !== undefined
            ? Promise.resolve(util_1.default.format(this.fulfilledMessage, deviceType))
            : Promise.reject(util_1.default.format(this.unfulfilledMessage, deviceType, supportedDevices.join(', ')));
    }
}
//# sourceMappingURL=create.js.map