"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Start = void 0;
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const AndroidEnvironmentRequirements_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/AndroidEnvironmentRequirements");
const AndroidUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/AndroidUtils");
const Common_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/Common");
const CommonUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/CommonUtils");
const IOSEnvironmentRequirements_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/IOSEnvironmentRequirements");
const IOSUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/IOSUtils");
const Requirements_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/Requirements");
const util_1 = __importDefault(require("util"));
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/lwc-dev-mobile', 'device-start');
const LWC_DEV_MOBILE = 'lwc-dev-mobile';
class Start extends command_1.SfdxCommand {
    constructor() {
        super(...arguments);
        this.examples = [
            `sfdx force:lightning:local:device:start -p iOS -t MySimulator`,
            `sfdx force:lightning:local:device:create -p Android -t MyEmulator -w`
        ];
        this.platform = '';
        this.target = '';
        this.writableSystem = false;
        this._requirements = {};
    }
    async run() {
        try {
            await this.init();
        }
        catch (error) {
            if (error instanceof core_1.SfdxError) {
                const sfdxError = error;
                sfdxError.actions = this.examples;
                throw sfdxError;
            }
            throw error;
        }
        this.logger.info(`Device Start command invoked for ${this.flags.platform}`);
        this.platform = this.flags.platform;
        this.target = this.flags.target;
        this.writableSystem = this.flags.writablesystem;
        return Requirements_1.RequirementProcessor.execute(this.commandRequirements).then(() => {
            this.logger.info('Setup requirements met, continuing with Device Start');
            return this.executeDeviceStart();
        });
    }
    get commandRequirements() {
        if (Object.keys(this._requirements).length === 0) {
            const requirements = {};
            requirements.setup = Common_1.CommandLineUtils.platformFlagIsAndroid(this.flags.platform)
                ? new AndroidEnvironmentRequirements_1.AndroidEnvironmentRequirements(this.logger, this.flags.apilevel)
                : new IOSEnvironmentRequirements_1.IOSEnvironmentRequirements(this.logger);
            this._requirements = requirements;
        }
        return this._requirements;
    }
    async init() {
        if (this.logger) {
            return Promise.resolve();
        }
        return super
            .init()
            .then(() => core_1.Logger.child('force:lightning:local:device:start', {}))
            .then((logger) => {
            this.logger = logger;
            return Promise.resolve();
        });
    }
    async executeDeviceStart() {
        const isAndroid = Common_1.CommandLineUtils.platformFlagIsAndroid(this.platform);
        return isAndroid
            ? this.executeAndroidDeviceStart()
            : this.executeIOSDeviceStart();
    }
    async executeAndroidDeviceStart() {
        return AndroidUtils_1.AndroidUtils.hasEmulator(this.target)
            .then((hasEmulator) => {
            if (!hasEmulator) {
                return Promise.reject(new core_1.SfdxError(util_1.default.format(messages.getMessage('error:targetDoesNotExistDescription'), this.target), LWC_DEV_MOBILE));
            }
            else {
                CommonUtils_1.CommonUtils.startCliAction(messages.getMessage('deviceStartAction'), util_1.default.format(messages.getMessage('deviceStartStatus'), this.target));
                return AndroidUtils_1.AndroidUtils.startEmulator(this.target, this.writableSystem, false);
            }
        })
            .then((actualPort) => {
            CommonUtils_1.CommonUtils.stopCliAction(util_1.default.format(messages.getMessage('deviceStartSuccessStatusAndroid'), this.target, actualPort, this.writableSystem));
            return Promise.resolve();
        });
    }
    async executeIOSDeviceStart() {
        let simDeviceUDID = '';
        return IOSUtils_1.IOSUtils.getSimulator(this.target)
            .then((simDevice) => {
            if (simDevice === null) {
                return Promise.reject(new core_1.SfdxError(util_1.default.format(messages.getMessage('error:targetDoesNotExistDescription'), this.target), LWC_DEV_MOBILE));
            }
            else {
                simDeviceUDID = simDevice.udid;
                return Promise.resolve();
            }
        })
            .then(() => {
            CommonUtils_1.CommonUtils.startCliAction(messages.getMessage('deviceStartAction'), util_1.default.format(messages.getMessage('deviceStartStatus'), `${this.target} (${simDeviceUDID})`));
            return IOSUtils_1.IOSUtils.bootDevice(simDeviceUDID, false);
        })
            .then(() => IOSUtils_1.IOSUtils.launchSimulatorApp())
            .then(() => {
            CommonUtils_1.CommonUtils.stopCliAction(util_1.default.format(messages.getMessage('deviceStartSuccessStatusIOS'), `${this.target} (${simDeviceUDID})`));
            return Promise.resolve();
        });
    }
}
exports.Start = Start;
Start.description = messages.getMessage('commandDescription');
Start.flagsConfig = {
    target: command_1.flags.string({
        char: 't',
        description: messages.getMessage('targetFlagDescription'),
        required: true,
        validate: (target) => {
            if (target && target.trim().length > 0) {
                return true;
            }
            else {
                throw new core_1.SfdxError(messages.getMessage('error:invalidTargetFlagsDescription'), LWC_DEV_MOBILE);
            }
        }
    }),
    writablesystem: command_1.flags.boolean({
        char: 'w',
        description: messages.getMessage('writablesystemFlagDescription'),
        required: false,
        default: false
    }),
    ...Common_1.CommandLineUtils.createFlagConfig(Common_1.FlagsConfigType.Platform, true)
};
//# sourceMappingURL=start.js.map